@@ -9,7 +9,7 @@
 /* slip : 16                             */
 /* lo   : 4                              */
 
-#ifdef LINUX
+#ifdef __linux__
 /*
 #define NETDEV_NR      3
 char *NETDEV[]={"lo","ppp","eth"};
@@ -21,7 +21,7 @@ int HEADSIZE []={0   ,0     ,14};		/* slip needs testing */
 #endif
 
 #ifdef SUNOS
-#define NETDEV_NR      3
+#define NETDEV_NR      2
 char *NETDEV[]={"le","hme"};
 int HEADSIZE[]={14  ,14};		/* ppp: 4 or 0 or nothing? */
 #endif
@@ -60,19 +60,24 @@ char *NETDEV[]={"ln"};
 int HEADSIZE[]={14};
 #endif
 
-#ifdef NETBSD
-#ifdef i386
+#ifdef __NetBSD__
+#if defined(i386) || defined(__amd64__)
 #define NETDEV_NR      21
 char *NETDEV[]={"ppp","ai","de","ec","ef","eg","el","en","ep","fe","fea","fpa","fxp","ix","iy","lc","le","ne","sm","tl","we"};
 int HEADSIZE[]={4    ,14  ,14  ,14  ,14  ,14  ,14  ,14  ,14  ,14  ,14   ,14   ,14   ,14  ,14  ,14  ,14  ,14  ,14  ,14  ,14  };
-#elif defined(sparc)
+#elif defined(__sparc__)
 #define NETDEV_NR      3
 char *NETDEV[]={"ppp","le","ie"};
 int HEADSIZE[]={4    ,14  ,14};
-#elif defined(amiga)
-#define ETH_DEV_NR      6
+#elif defined(__m68k__)
+#define NETDEV_NR      6
 char *NETDEV[]={"ppp","bah","ed","es","le","qn"};
 int HEADSIZE[]={4    ,14  ,14   ,14  ,14  ,14};
+#elif defined(__mips64) && defined(__MIPSEB__)
+/* Assume Octeon. */
+#define NETDEV_NR      1
+char *NETDEV[]={"cnmac"};
+int HEADSIZE[]={14};
 #else
 #error Unknown network devices for this NetBSD architecture.
 #endif
@@ -90,6 +95,15 @@ char *NETDEV[]={"le","sam"};
 int HEADSIZE[]={14  ,14   };
 #endif
 
+#ifdef __HAIKU__
+#define NETDEV_NR 4
+char *NETDEV[]={"loop","tun","tap","/dev/net/"};
+int HEADSIZE[]={0,     0,    14,   14};
+#endif
+
+#ifndef NETDEV_NR
+#error "Missing network device data for the current OS, please fix!"
+#endif
 /**** Global data **********************************************************/
 pcap_t *dev_desc;
 pcap_dumper_t *dev_dump;
@@ -151,11 +165,11 @@ char Plugin_Active[10];
 #ifdef INCLUDE_INTERFACE
 
 /**** shared memory pointers ************************************************/
-char *SHARED, *connection_data, *timing, *running_connections,
-						  *logged_connections;
+char *SHARED, *connection_data, *logged_connections;
+struct shared_conn_data *running_connections;
 int *LISTlength, *DATAlength, memory_id;
 unsigned int  *TCP_nr_of_packets, *ICMP_nr_of_packets, *UDP_nr_of_packets;
-extern unsigned int  *IP_nr_of_packets;
+unsigned int  *IP_nr_of_packets;
 unsigned long *TCP_bytes_in_packets, *UDP_bytes_in_packets;
 int *DESC_LEN;
 
