@@ -20,10 +20,11 @@
 #include "sn_resolv.h"
 
 /*** extern stuff ********/
-extern char *SHARED, *connection_data, *timing, *running_connections,
-                                                  *logged_connections;
+extern char *SHARED, *connection_data, *logged_connections;
+extern struct shared_conn_data *running_connections;
 extern int *LISTlength, *DATAlength, memory_id;
-extern unsigned int  *TCP_nr_of_packets, *ICMP_nr_of_packets, *UDP_nr_of_packets; unsigned int  *IP_nr_of_packets;
+extern unsigned int  *TCP_nr_of_packets, *ICMP_nr_of_packets, *UDP_nr_of_packets;
+extern unsigned int  *IP_nr_of_packets;
 extern unsigned long *TCP_bytes_in_packets, *UDP_bytes_in_packets;
 extern int *DESC_LEN;   /* For the connection desciption */
 
@@ -52,8 +53,13 @@ extern int INFO_WINDOW_X, INFO_WINDOW_Y, MASK_WINDOW_X, MASK_WINDOW_Y;
 extern int DATA_WINDOW_X, DATA_WINDOW_Y;
 
 
+/*** forward declarations ***/
+static void stop_logging (void);
+static void screen_exit (void);
+
+
 /*** Sreen operations ***/
-void init_screen (void)
+static void init_screen (void)
 {
 initscr();
 cbreak();
@@ -90,7 +96,7 @@ if( (COLS<80)||(LINES<18) )
 	exit(0);
 };
 
-void f_box_window (struct box_window *Win,
+static void f_box_window (struct box_window *Win,
                  int num_lines, int num_cols, int begy,int begx, int col_mode)
 /*  col_mode : color selection   */
 {
@@ -126,14 +132,12 @@ wnoutrefresh(Win->main_window);wnoutrefresh(Win->work_window);
 doupdate();
 }
 
-void data_window (struct box_window *Win, struct box_window *P_Win,
+static void data_window (struct box_window *Win, struct box_window *P_Win,
                  int num_lines, int num_cols, int begy,int begx,
-                 char *buffer, int listitem)
+                 struct shared_conn_data *conn, int listitem)
 {
 int i=0, j=0;
-struct shared_conn_data *conn;
 
-conn = (struct shared_conn_data *) buffer;
 while((j<listitem)&&(i<(CONNECTION_CAPACITY+1)))
 	{
   	if(conn[i].connection[0]!=0)
@@ -164,12 +168,10 @@ wnoutrefresh(Win->main_window);wnoutrefresh(Win->work_window);
 doupdate();
 }
 
-void data_device (char *buffer, int listitem)
+static void data_device (struct shared_conn_data *conn, int listitem)
 {
 int i=0, j=0;
-struct shared_conn_data *conn;
 
-conn = (struct shared_conn_data *) buffer;
 while((j<listitem)&&(i<(CONNECTION_CAPACITY+1)))
 	{
   	if(conn[i].connection[0]!=0)
@@ -182,10 +184,11 @@ if(i>=CONNECTION_CAPACITY+1) return;
 strcpy(log_conn->log_enter, conn[i].connection);
 }
 
-void mask_status (struct box_window *Work_win)
+static void mask_status (struct box_window *Work_win)
 {
-unsigned char *ad;
+const char *ad;
 int i;
+char ipv4_buf[sizeof("000.000.000.000")];
 
 wmove(Work_win->work_window,0,1);
 for(i=0;i<2;i++)
@@ -193,16 +196,27 @@ for(i=0;i<2;i++)
 	whline(Work_win->work_window,' ',COLS-2);}
 wmove(Work_win->work_window,0,1);
 wprintw(Work_win->work_window,"Source IP     : ");
-ad=&(mask->source_ip);
-if(mask->source_ip==0)
- 	wprintw(Work_win->work_window,"All");
-else wprintw(Work_win->work_window,"%u.%u.%u.%u",ad[0],ad[1],ad[2],ad[3]);
+/* Both source_ip and destination_ip are in network byte order. */
+ad = mask->source_ip ?
+	inet_ntop(AF_INET,&mask->source_ip,ipv4_buf,sizeof(ipv4_buf)) :
+	"All";
+if(!ad)
+{
+	perror("inet_ntop");
+	exit(1);
+}
+wprintw(Work_win->work_window,"%s",ad);
 wmove(Work_win->work_window,1,1);
 wprintw(Work_win->work_window,"Destination IP: ");
-ad=&(mask->destination_ip);
-if(mask->destination_ip==0)
- 	wprintw(Work_win->work_window,"All");
-else wprintw(Work_win->work_window,"%u.%u.%u.%u",ad[0],ad[1],ad[2],ad[3]);
+ad = mask->source_ip ?
+	inet_ntop(AF_INET,&mask->destination_ip,ipv4_buf,sizeof(ipv4_buf)) :
+	"All";
+if(!ad)
+{
+	perror("inet_ntop");
+	exit(1);
+}
+wprintw(Work_win->work_window,"%s",ad);
 wmove(Work_win->work_window,0,35);
 wprintw(Work_win->work_window,"Source PORT     : ");
 if(mask->source_port==0)
@@ -218,14 +232,12 @@ wnoutrefresh(Work_win->work_window);
 doupdate();
 }
 
-void fill_box_window (struct box_window *Work_win, char *buffer,
+static void fill_box_window (struct box_window *Work_win, struct shared_conn_data *conn,
                       int begin_item, int boxlen, int rowlen)
                                                  /* 0 is the first item  */
 {
 int i=0, j=0, line=0;
-struct shared_conn_data *conn;
 
-conn = (struct shared_conn_data *) buffer;
 while((j<begin_item)&&(i<(CONNECTION_CAPACITY+1)))
 	{
   	if(conn[i].connection[0]!=0)
@@ -269,18 +281,15 @@ for(i=line;i<boxlen;i++)
 wnoutrefresh(Work_win->work_window);
 }
 
-void point_item (struct box_window *Work_win, char *buffer,
+static void point_item (struct box_window *Work_win, struct shared_conn_data *conn,
                  int item, int begin_item, int boxlen, int rowlen)
 {
 int i=0, j=0;
-struct shared_conn_data *conn;
-
 
 if(item<0) return;      /* POINTpos   0 = first item   -1 = no items */
                         /* LISTlength 0 = 1            -1 = no items */
                         /* DANGER - there should always be >=        */
                         /*          connections than 'item'          */
-conn = (struct shared_conn_data *) buffer;
 while((j<item)&&(i<(CONNECTION_CAPACITY+1)))
 	{
 	if(conn[i].connection[0] !=0)
@@ -361,7 +370,7 @@ if(PACKET_INFO==1)
 doupdate();
 }
 
-void menu_line (void)
+static void menu_line (void)
 {
 int i;
 
@@ -412,7 +421,7 @@ char *input_field(char *string, char *input, int flag)
 	return input;
 }
 
-void exec_mask (void)
+static void exec_mask (void)
 {
 LISTpos=0;
 POINTpos=-1;             /* otherwise we get never ending loop */
@@ -424,7 +433,7 @@ forced_refresh();
 
 /* signaling */
 
-void sig_blocking(char on_off, int sig)
+static void sig_blocking(char on_off, int sig)
 {
 sigset_t set;
 
@@ -434,7 +443,7 @@ if(on_off==1)
 else	{sigprocmask(SIG_UNBLOCK,&set,NULL);}
 }
 
-void set_signal (int signum, sig_hand new_action)
+static void set_signal (int signum, sig_hand new_action)
 {
 struct sigaction new_sigusr;
 sigset_t sig_mask;
@@ -448,14 +457,13 @@ new_sigusr.sa_flags=0;
 sigaction(signum,&new_sigusr,NULL);
 }
 
-void interaction (int sig)              /* invoked when data arrives */
+static void interaction (int sig)              /* invoked when data arrives */
 {
 int i;
-struct shared_conn_data *conn;
+struct shared_conn_data *conn = running_connections;
 
 
 /* timeout increase */
-conn = (struct shared_conn_data *) running_connections;
 for(i=0;i<CONNECTION_CAPACITY;i++)
   	if(conn[i].connection[0]!=0)
 		conn[i].timeout+=1;
@@ -483,16 +491,16 @@ forced_refresh();
 set_signal(SIGUSR1,interaction);
 }
 
-void packet_info_handler (int signum)
+static void packet_info_handler (int signum)
 {
 #ifdef DEBUG
 		debug_msg("ALARM RANG");
 #endif
 screen_busy=1;
-mvwprintw(packets_box.work_window,0,1,"IP packets/sec.  : % 12u",(*IP_nr_of_packets)/INFO_TIMER);
-mvwprintw(packets_box.work_window,1,1,"TCP packets/sec. : % 12u",(*TCP_nr_of_packets)/INFO_TIMER);
-mvwprintw(packets_box.work_window,2,1,"ICMP packets/sec.: % 12u",(*ICMP_nr_of_packets)/INFO_TIMER);
-mvwprintw(packets_box.work_window,3,1,"UDP packets/sec. : % 12u",(*UDP_nr_of_packets)/INFO_TIMER);
+mvwprintw(packets_box.work_window,0,1,"IP packets/sec.  : %12u",(*IP_nr_of_packets)/INFO_TIMER);
+mvwprintw(packets_box.work_window,1,1,"TCP packets/sec. : %12u",(*TCP_nr_of_packets)/INFO_TIMER);
+mvwprintw(packets_box.work_window,2,1,"ICMP packets/sec.: %12u",(*ICMP_nr_of_packets)/INFO_TIMER);
+mvwprintw(packets_box.work_window,3,1,"UDP packets/sec. : %12u",(*UDP_nr_of_packets)/INFO_TIMER);
 mvwprintw(packets_box.work_window,4,1,"bytes/sec. (TCP) : % 12ld",(*TCP_bytes_in_packets)/INFO_TIMER);
 mvwprintw(packets_box.work_window,5,1,"bytes/sec. (UDP) : % 12ld",(*UDP_bytes_in_packets)/INFO_TIMER);
 screen_busy=0;
@@ -515,7 +523,7 @@ void child_exit (void)
 kill(Pid,SIGKILL);
 };
 
-void screen_exit (void)
+static void screen_exit (void)
 {
 endwin();
 /* next line added by Edward Betts <edward@debian.org>, should not be needed
@@ -532,7 +540,7 @@ if(shmctl(memory_id,IPC_RMID,0)<0)
 
 /* Some other stuff */
 
-void stop_logging (void)
+static void stop_logging (void)
 {
 LOGGING=0;
 log_conn->log_enter[0]=0;
@@ -541,7 +549,7 @@ if(logging_device==NULL)
 forced_refresh();
 }
 
-void stop_packet_info (void)
+static void stop_packet_info (void)
 {
 PACKET_INFO=0;
 alarm(0);
@@ -549,14 +557,12 @@ delwin(packets_box.work_window), delwin(packets_box.main_window);
 forced_refresh();
 }
 
-int add_itemlist(char *buffer, char *string, char *desc)
+int add_itemlist(struct shared_conn_data *conn, char *string, char *desc)
 {
 int i, to_help, to_item;
-struct shared_conn_data *conn;
 
 /*invoked every time a packet comes in */
 
-conn = (struct shared_conn_data *) buffer;
 for(i=0;i<CONNECTION_CAPACITY;i++)
 	if(strcmp( conn[i].connection, string)==0)
 		{
@@ -589,12 +595,10 @@ conn[to_item].timeout=0;
 return to_item;
 }
 
-int del_itemlist(char *buffer, char *string)
+int del_itemlist(struct shared_conn_data *conn, char *string)
 {
 int i;
-struct shared_conn_data *conn;
 
-conn = (struct shared_conn_data *) buffer;
 for(i=0;i<CONNECTION_CAPACITY;i++)
 	if(strcmp( conn[i].connection, string)==0)
     		{
@@ -615,9 +619,8 @@ void clear_shared_mem(char mode)
 			/*             keep packet count */
 {
 int i;
-struct shared_conn_data *conn;
+struct shared_conn_data *conn = running_connections;
 
-*timing=1;
 *DATAlength=0;
 *LISTlength=-1;
 if(mode==0)
@@ -630,25 +633,21 @@ if(mode==0)
   }
 
 log_conn->log_enter[0]=0;
-conn = (struct shared_conn_data *) running_connections;
 for(i=0;i<CONNECTION_CAPACITY;i++)
 	{
 	conn[i].connection[0]=0;
 	conn[i].desc[0]=0;
 	conn[i].timeout=0;
 	}
-*timing=0;
 };
 
-void create_arguments(char *esource, char *es_port, char *edest,
-				        char *ed_port, char *buffer, int item)
+static void create_arguments(char *esource, char *es_port, char *edest,
+				        char *ed_port, struct shared_conn_data *conn, int item)
 {
 char e_dummy[CONN_NAMELEN];
 int i=0, j=0;
-struct shared_conn_data *conn;
 
 if(item<0) return;
-conn = (struct shared_conn_data *) buffer;
 while((j<item)&&(i<(CONNECTION_CAPACITY+1)))
 	{
   	if(conn[i].connection[0] !=0)
