@@ -44,10 +44,13 @@
 
 #include "sniffit.h"		/* definition of functions  */
 
+/*** forward declarations ***/
+static void delete_dynam (char *, char, char);
+
 static char Copyright[] =
 "Sniffit - Brecht Claerhout - Copyright 1996-98";
 
-void quit (char *prog_name)		/* Learn to use the program */
+static void quit (char *prog_name)		/* Learn to use the program */
 {
   printf (
 	   "usage: %s [-xdabvnN] [-P proto] [-A char] [-p port] [(-r|-R) recordfile]\n"
@@ -99,7 +102,7 @@ void quit (char *prog_name)		/* Learn to use the program */
 void close_dumpfile(void) {pcap_dump_close(dev_dump);};
 void close_pcapdev(void) {pcap_close(dev_desc);};
 
-void my_exit (void)
+static void my_exit (int signo)
 {
   fflush(NULL);
   printf("Graceful shutdown...\n");
@@ -129,7 +132,7 @@ char *strlower (char *string)
   return string;
 }
 
-void start_plugin (int PL_nr, struct Plugin_data *PL_d)
+static void start_plugin (int PL_nr, struct Plugin_data *PL_d)
 {
   switch (PL_nr)
     {
@@ -195,7 +198,7 @@ void start_plugin (int PL_nr, struct Plugin_data *PL_d)
     }
 }
 
-void
+static void
 reset_all (void)
 {
   start_dynam = NULL;
@@ -204,7 +207,7 @@ reset_all (void)
 
 /* if do_file == 0, then don't handle the files */
 /* this is for the global logfile option        */
-struct file_info *
+static struct file_info *
 add_dynam (char *file, char ptype, char do_file,
 	   _32_bit cur_seq, int len)
 {
@@ -281,7 +284,7 @@ add_dynam (char *file, char ptype, char do_file,
   return dummy_pointer;
 }
 
-void
+static void
 delete_dynam (char *file, char ptype, char do_file)
 {
   struct file_info *search_pointer;
@@ -322,7 +325,7 @@ delete_dynam (char *file, char ptype, char do_file)
 }
 
 /* returns NULL on failure */
-struct file_info *
+static struct file_info *
 search_dynam (char *file, char ptype)
 {
   struct file_info *search_pointer;
@@ -354,9 +357,9 @@ search_dynam (char *file, char ptype)
 }
 
 /* Type 0: TELNET  */
-void
+static void
 record_buf (struct file_info *dummy_pointer, _32_bit cur_seq_nr,
-	    char *data, int len, int type)
+	    const char *data, int len, int type)
 {
   int i, j, noloop = 0;
 
@@ -400,7 +403,7 @@ record_buf (struct file_info *dummy_pointer, _32_bit cur_seq_nr,
 #endif
 }
 
-void
+static void
 sb_shift (struct file_info *dummy_pointer)
 {
   int i, j;
@@ -409,9 +412,9 @@ sb_shift (struct file_info *dummy_pointer)
     dummy_pointer->scroll_buf[i - 1] = dummy_pointer->scroll_buf[i];
 }
 
-void
+static void
 sbuf_update (struct file_info *dummy_pointer, _32_bit cur_seq_nr,
-	     char *data, int len)
+	     const char *data, int len)
 {
   int i;
 
@@ -439,7 +442,7 @@ sbuf_update (struct file_info *dummy_pointer, _32_bit cur_seq_nr,
 #endif
 }
 
-void
+static void
 print_iphead (struct IP_header *iphead, char icmp_or_plain)
 {
   int dummy;
@@ -474,9 +477,9 @@ print_iphead (struct IP_header *iphead, char icmp_or_plain)
   printf ("\n");
 }
 
-int
+static int
 check_packet (_32_bit ipaddr,
-	      const struct packetheader *p_header,
+	      const struct pcap_pkthdr *p_header,
 	      const unsigned char *sp,
 	      char *file,
 	      char *file2,
@@ -496,7 +499,7 @@ check_packet (_32_bit ipaddr,
   unsigned int prior;
   char selected;
   int i;			/* Wildcard stuff */
-  unsigned char *str_IP;
+  char *str_IP;
   unsigned int n_s;		/* strlen short notation for wc bugfix */
 
   proto = unwrap_packet (sp, info);
@@ -1068,9 +1071,9 @@ if((info->FRAG_nf!=0)||(proto==TCP_FRAG_HEAD))
 }
 
 /* Default Processing of packets */
-pcap_handler
+static void
 packethandler (unsigned char *ipaddrpoint,
-	       const struct packetheader * p_header,
+	       const struct pcap_pkthdr * p_header,
 	       const unsigned char *sp)
 {
   char filename[50], filename2[50], header[SNAPLEN];
@@ -1094,19 +1097,19 @@ packethandler (unsigned char *ipaddrpoint,
   finish = check_packet (ipaddr, p_header, sp, filename, filename2, &info, header, SNIFMODE);
 
   if (finish == DROP_PACKET)
-    return 0;			/* Packet is broken */
+    return;			/* Packet is broken */
 
   if( (PROTOCOLS&F_IP)&&((PROTOCOLS&F_TCP)==0))
     memcpy (&iphead, (sp + PROTO_HEAD), sizeof (struct IP_header)),
       print_iphead (&iphead, 0);
 
   if (finish == DONT_EXAMINE)
-    return 0;			/* Packet is not for us */
+    return;			/* Packet is not for us */
 
   if(DUMPMODE==8)               /* Recording */
     {
     pcap_dump((unsigned char *) dev_dump, p_header, sp);
-    return 1;
+    return;
     }
 
   if((PROTOCOLS & F_IP)&&(PROTOCOLS & F_TCP)&&(finish<10))
@@ -1152,9 +1155,9 @@ packethandler (unsigned char *ipaddrpoint,
       dummy = ntohs (tcphead.offset_flag);
       printf ("\n");
       printf ("TCP Packet ID (from_IP.port-to_IP.port): %s\n", filename);
-      printf ("   SEQ (hex): %lX   ", ntohl (tcphead.seq_nr));
+      printf ("   SEQ (hex): %X   ", ntohl (tcphead.seq_nr));
       if (dummy & ACK)
-	printf ("ACK (hex): %lX\n", ntohl (tcphead.ACK_nr));
+	printf ("ACK (hex): %X\n", ntohl (tcphead.ACK_nr));
       printf ("   FLAGS: %c%c%c%c%c%c",
 	      (dummy & URG) ? 'U' : '-', (dummy & ACK) ? 'A' : '-',
 	      (dummy & PSH) ? 'P' : '-', (dummy & RST) ? 'R' : '-',
@@ -1187,11 +1190,11 @@ packethandler (unsigned char *ipaddrpoint,
 	  if (status == 0)
 	    {
 	      if (finish == TCP_FINISH)
-		return 1;
+		return;
 	      /* there was never data transmitted */
 	      /* seq_nr & datalen not important here yet */
 	      if ((dummy_pointer = add_dynam (filename, TCP, 1, 0, 0)) == NULL)
-		return 1;
+		return;
 	    }
 	  f = dummy_pointer->f;
 
@@ -1257,7 +1260,7 @@ packethandler (unsigned char *ipaddrpoint,
 	  printf ("\nYou mixed incompatible options!\n");
 	  exit (1);
 	}
-      return 1;
+      return;
     }
 
   if ((finish < 10) && (LOGPARAM != 0))		/*  TCP packet - logfile   */
@@ -1337,7 +1340,7 @@ packethandler (unsigned char *ipaddrpoint,
 	  break;
 	}
       printf ("\n");
-      return 1;
+      return;
     }
   if (finish < 30)		/* nothing yet */
     {
@@ -1370,14 +1373,14 @@ packethandler (unsigned char *ipaddrpoint,
 	  printf ("\nImpossible error! Sniffer Heartattack!\n");
 	  exit (0);
 	}
-      return 1;
+      return;
     }
 }
 
 
 #ifdef INCLUDE_INTERFACE	/* Interactive packethandling */
-int
-check_mask (const struct packetheader *p_header,
+static int
+check_mask (const struct pcap_pkthdr *p_header,
 	    const unsigned char *sp,
 	    char *conn_name, char *conn_name2, char *desc_string,
 	    struct unwrap *info)
@@ -1468,9 +1471,9 @@ if(INTERACTIVE_EXTEND==1)
   return TCP_EXAMINE;		/* interprete packet */
 }
 
-pcap_handler
-interactive_packethandler (char *dummy,
-			   const struct packetheader * p_header,
+static void
+interactive_packethandler (unsigned char *dummy,
+			   const struct pcap_pkthdr * p_header,
 			   const unsigned char *sp)
 {
   char conn_name[CONN_NAMELEN], conn_name2[CONN_NAMELEN];
@@ -1483,9 +1486,9 @@ interactive_packethandler (char *dummy,
 
   finish = check_mask (p_header, sp, conn_name, conn_name2, desc_string, &info);
   if (finish == DROP_PACKET)
-    return 0;			/* Packet is broken */
+    return;			/* Packet is broken */
   if (finish == DONT_EXAMINE)
-    return 0;			/* Packet is not for us */
+    return;			/* Packet is not for us */
 
   if (finish != TCP_FINISH)	/* finish: already logged, or to short to add */
     add_itemlist (running_connections, conn_name, desc_string);
@@ -1513,9 +1516,9 @@ int main (int argc, char *argv[])
 {
   char *dev, forced_dev[20], buffer[SNAPLEN];
   char ebuf[PCAP_ERRBUF_SIZE];
-  unsigned char *DUMPfile;           /* file used for packed logging */
+  char *DUMPfile;           /* file used for packed logging */
   int c, i;
-  unsigned long memsize;
+  size_t memsize;
   _32_bit ipaddr;
   int flag = 0, doboth = 0, FORCE_DEV = 0, SUPPORTED = 0;
   extern char *optarg;
@@ -1576,12 +1579,12 @@ int main (int argc, char *argv[])
         case 'R':                    /* recording (not mixable) */
           if(DUMPMODE) quit(argv[0]);
           DUMPMODE=8;
-          DUMPfile=(unsigned char *)optarg;
+          DUMPfile=optarg;
           break;
         case 'r':
           if(DUMPMODE&8) quit(argv[0]);
           DUMPMODE|=16;              /* reading (mixable) */
-          DUMPfile=(unsigned char *)optarg;
+          DUMPfile=optarg;
           break;
         case 'x':
 	  if(DUMPMODE&8) quit(argv[0]);
@@ -1626,13 +1629,13 @@ int main (int argc, char *argv[])
 	  break;
 	case 'P':
 	  strlower (optarg);
-	  if (strstr (optarg, "tcp"))
+	  if (strstr (optarg, "tcp") || strstr (optarg, "TCP"))
 	    PROTOCOLS |= F_TCP;
-	  if (strstr (optarg, "icmp"))
+	  if (strstr (optarg, "icmp") || strstr (optarg, "ICMP"))
 	    PROTOCOLS |= F_ICMP;
-	  if (strstr (optarg, "udp"))
+	  if (strstr (optarg, "udp") || strstr (optarg, "UDP"))
 	    PROTOCOLS |= F_UDP;
-	  if (strstr (optarg, "ip"))
+	  if (strstr (optarg, "ip") || strstr (optarg, "IP"))
 	    PROTOCOLS |= F_IP;
 	  break;
 	case 's':
@@ -1771,8 +1774,14 @@ int main (int argc, char *argv[])
     }
   else
     {
-    if((dev = pcap_lookupdev (ebuf)) == NULL)
+    pcap_if_t *alldevsp;
+    if (pcap_findalldevs (&alldevsp, ebuf) == PCAP_ERROR)
        fprintf (stderr,"%s\n",ebuf), exit (1);
+    if (alldevsp == NULL)
+       fprintf (stderr, "No network devices available.\n"), exit (1);
+    if ((dev = strdup (alldevsp[0].name)) == NULL)
+       perror ("strdup"), exit (1);
+    pcap_freealldevs (alldevsp);
     }
 
   for (i = 0; i < NETDEV_NR; i++)
@@ -1792,49 +1801,51 @@ int main (int argc, char *argv[])
     }
 
   if(!(DUMPMODE&16))
+    {
     if ((dev_desc = pcap_open_live (dev, SNAPLEN, 1, MSDELAY, ebuf)) == NULL)
       fprintf (stderr,"%s\n",ebuf), exit (0);
     else
       exit_func(close_pcapdev);
+    }
   if(FORCE_DEV!=0) {free(dev);}  /* no longer needed  */
 
-#ifdef PLUGIN0_INIT()
+#ifdef PLUGIN0_INIT
 if (Plugin_Active[0] == 1)
   PLUGIN0_INIT ();
 #endif
-#ifdef PLUGIN1_INIT()
+#ifdef PLUGIN1_INIT
 if (Plugin_Active[1] == 1)
   PLUGIN1_INIT ();
 #endif
-#ifdef PLUGIN2_INIT()
+#ifdef PLUGIN2_INIT
 if (Plugin_Active[2] == 1)
   PLUGIN2_INIT ();
 #endif
-#ifdef PLUGIN3_INIT()
+#ifdef PLUGIN3_INIT
 if (Plugin_Active[3] == 1)
   PLUGIN3_INIT ();
 #endif
-#ifdef PLUGIN4_INIT()
+#ifdef PLUGIN4_INIT
 if (Plugin_Active[4] == 1)
   PLUGIN4_INIT ();
 #endif
-#ifdef PLUGIN5_INIT()
+#ifdef PLUGIN5_INIT
 if (Plugin_Active[5] == 1)
   PLUGIN5_INIT ();
 #endif
-#ifdef PLUGIN6_INIT()
+#ifdef PLUGIN6_INIT
 if (Plugin_Active[6] == 1)
   PLUGIN6_INIT ();
 #endif
-#ifdef PLUGIN7_INIT()
+#ifdef PLUGIN7_INIT
 if (Plugin_Active[7] == 1)
   PLUGIN7_INIT ();
 #endif
-#ifdef PLUGIN8_INIT()
+#ifdef PLUGIN8_INIT
 if (Plugin_Active[8] == 1)
   PLUGIN8_INIT ();
 #endif
-#ifdef PLUGIN9_INIT()
+#ifdef PLUGIN9_INIT
 if (Plugin_Active[9] == 1)
   PLUGIN9_INIT ();
 #endif
@@ -1842,7 +1853,7 @@ if (Plugin_Active[9] == 1)
 #ifdef INCLUDE_INTERFACE
   if (SNIFMODE == INTERACTIVE)
     {
-      memsize = sizeof (int) + sizeof (int) + LENGTH_OF_INTERPROC_DATA +
+      memsize = sizeof (int) + LENGTH_OF_INTERPROC_DATA +
         sizeof (int) + sizeof (struct snif_mask) +
         sizeof (struct shared_logged_conn) +
         (CONNECTION_CAPACITY * sizeof (struct shared_conn_data)) +
@@ -1862,23 +1873,23 @@ if (Plugin_Active[9] == 1)
 	  exit (0);
 	};
       printf ("Entering Shared memory at %p\n", SHARED);
-      printf ("Shared %d\n", memsize);
+      printf ("Shared %zu\n", memsize);
 
-      timing = SHARED;		/* set all pointers */
-      DATAlength = timing + sizeof (int);
-      connection_data = DATAlength + sizeof (int);
-      LISTlength = connection_data + LENGTH_OF_INTERPROC_DATA;
-      mask = LISTlength + sizeof (int);
-      logged_connections = mask + sizeof (struct snif_mask);
+      /* set all pointers */
+      DATAlength = (int *) SHARED;
+      connection_data = (char *) (DATAlength + 1);
+      LISTlength = (int *) (connection_data + LENGTH_OF_INTERPROC_DATA);
+      mask = (struct snif_mask *)((char *)LISTlength + sizeof(int));
+      logged_connections = (char *)mask + sizeof(struct snif_mask);
       log_conn = (struct shared_logged_conn *) logged_connections;
-      running_connections = logged_connections + sizeof (struct shared_logged_conn);
-      TCP_nr_of_packets = running_connections + (sizeof (struct shared_conn_data) * CONNECTION_CAPACITY);
-      TCP_bytes_in_packets = TCP_nr_of_packets + sizeof (int);
-      ICMP_nr_of_packets = TCP_bytes_in_packets + sizeof (unsigned long);
-      UDP_nr_of_packets = ICMP_nr_of_packets + sizeof (int);
-      UDP_bytes_in_packets = UDP_nr_of_packets + sizeof (int);
-      IP_nr_of_packets = UDP_bytes_in_packets + sizeof (unsigned long);
-      DESC_LEN = IP_nr_of_packets + sizeof (int);
+      running_connections = (struct shared_conn_data *)((char *)logged_connections + sizeof(struct shared_logged_conn));
+      TCP_nr_of_packets = (int *)((char *)running_connections + sizeof(struct shared_conn_data) * CONNECTION_CAPACITY);
+      TCP_bytes_in_packets = (unsigned long *)((char *)TCP_nr_of_packets + sizeof(int));
+      ICMP_nr_of_packets = (unsigned int *)((char *)TCP_bytes_in_packets + sizeof(unsigned long));
+      UDP_nr_of_packets = (unsigned int *)(ICMP_nr_of_packets + 1);
+      UDP_bytes_in_packets = (unsigned long *)((char *)UDP_nr_of_packets + sizeof(unsigned int));
+      IP_nr_of_packets = (unsigned int *)((char *)UDP_bytes_in_packets + sizeof(unsigned long));
+      DESC_LEN = (int *)(IP_nr_of_packets + 1);
       clear_shared_mem (0);
 
       *DESC_LEN = 10;             /* not necessary, but for security (eliminate very unlikely races) */
